@@ -1,5 +1,6 @@
 import copy
 import typing
+import warnings
 
 import numpy as np
 
@@ -169,7 +170,9 @@ class AbstractEPM(object):
         """
         raise NotImplementedError
 
-    def predict(self, X: np.ndarray) -> typing.Tuple[np.ndarray, np.ndarray]:
+    def predict(self, X: np.ndarray,
+                cov_return_type: typing.Optional[str] = 'diagonal_cov') \
+            -> typing.Tuple[np.ndarray, typing.Optional[np.ndarray]]:
         """
         Predict means and variances for given X.
 
@@ -177,13 +180,20 @@ class AbstractEPM(object):
         ----------
         X : np.ndarray of shape = [n_samples, n_features (config + instance features)]
             Training samples
+        cov_return_type: typing.Optional[str]
+            Specifies what to return along with the mean. (Applies to only Gaussian Process for now)
+            Can take 4 values: [None, diagonal_std, diagonal_cov, full_cov]
+            * None - only mean is returned
+            * diagonal_std - standard deviation at test points is returned
+            * diagonal_cov - diagonal of the covariance matrix is returned
+            * full_cov - whole covariance matrix between the test points is returned
 
         Returns
         -------
         means : np.ndarray of shape = [n_samples, n_objectives]
             Predictive mean
-        vars : np.ndarray  of shape = [n_samples, n_objectives]
-            Predictive variance
+        vars : None or np.ndarray of shape = [n_samples, n_objectives] or [n_samples, n_samples]
+            Predictive variance or standard deviation
         """
         if len(X.shape) != 2:
             raise ValueError('Expected 2d array, got %dd array!' % len(X.shape))
@@ -202,16 +212,20 @@ class AbstractEPM(object):
         if X.shape[1] != len(self.types):
             raise ValueError('Rows in X should have %d entries but have %d!' % (len(self.types), X.shape[1]))
 
-        mean, var = self._predict(X)
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', 'Predicted variances smaller than 0. Setting those variances to 0.')
+            mean, var = self._predict(X, cov_return_type)
 
         if len(mean.shape) == 1:
             mean = mean.reshape((-1, 1))
-        if len(var.shape) == 1:
+        if var is not None and len(var.shape) == 1:
             var = var.reshape((-1, 1))
 
         return mean, var
 
-    def _predict(self, X: np.ndarray) -> typing.Tuple[np.ndarray, np.ndarray]:
+    def _predict(self, X: np.ndarray,
+                 cov_return_type: typing.Optional[str] = 'diagonal_cov') \
+            -> typing.Tuple[np.ndarray, typing.Optional[np.ndarray]]:
         """
         Predict means and variances for given X.
 
@@ -219,13 +233,15 @@ class AbstractEPM(object):
         ----------
         X : np.ndarray
             [n_samples, n_features (config + instance features)]
+        cov_return_type: typing.Optional[str]
+            Specifies what to return along with the mean. Refer ``predict()`` for more information.
 
         Returns
         -------
         means : np.ndarray of shape = [n_samples, n_objectives]
             Predictive mean
-        vars : np.ndarray  of shape = [n_samples, n_objectives]
-            Predictive variance
+        vars : None or np.ndarray of shape = [n_samples, n_objectives] or [n_samples, n_samples]
+            Predictive variance or standard deviation
         """
         raise NotImplementedError()
 
@@ -257,6 +273,8 @@ class AbstractEPM(object):
         if self.instance_features is None or \
                 len(self.instance_features) == 0:
             mean, var = self.predict(X)
+            assert var is not None  # please mypy
+
             var[var < self.var_threshold] = self.var_threshold
             var[np.isnan(var)] = self.var_threshold
             return mean, var
@@ -269,6 +287,7 @@ class AbstractEPM(object):
             X_ = np.hstack(
                 (np.tile(x, (n_instances, 1)), self.instance_features))
             means, vars = self.predict(X_)
+            assert vars is not None  # please mypy
             # VAR[1/n (X_1 + ... + X_n)] =
             # 1/n^2 * ( VAR(X_1) + ... + VAR(X_n))
             # for independent X_1 ... X_n
