@@ -23,9 +23,6 @@ class LHDesign(InitialDesign):
         List of configurations to be evaluated
         Don't pass configs to the constructor;
         otherwise factorial design is overwritten
-    intensifier
-    runhistory
-    aggregate_func
     """
 
     def _select_configurations(self) -> typing.List[Configuration]:
@@ -37,19 +34,18 @@ class LHDesign(InitialDesign):
             initial incumbent configuration
         """
 
-        cs = self.scenario.cs
-        params = cs.get_hyperparameters()
+        params = self.cs.get_hyperparameters()
 
         # seeding of lhd design
-        np.random.seed(self.rng.randint(1,2*20))
+        np.random.seed(self.rng.randint(1, 2 * 20))
 
         constants = 0
         for p in params:
             if isinstance(p, Constant):
                 constants += 1
 
-        lhd = pyDOE.lhs(n=len(params)-constants, samples=self.init_budget)
+        lhd = pyDOE.lhs(n=len(params) - constants, samples=self.init_budget)
 
         return self._transform_continuous_designs(design=lhd,
                                                   origin='LHD',
-                                                  cs=cs)
+                                                  cs=self.cs)
