@@ -22,9 +22,10 @@ __license__ = "3-clause BSD"
 
 class IntensifierStage(Enum):
     """Class to define different stages of intensifier"""
+    RUN_FIRST_CONFIG = 0  # to replicate the old initial design
     RUN_INCUMBENT = 1  # Lines 3-7
     RUN_CHALLENGER = 2  # Lines 8-17
-    RUN_DEFAULT = 3
+    RUN_BASIS = 3
 
 
 class Intensifier(AbstractRacer):
@@ -112,14 +113,15 @@ class Intensifier(AbstractRacer):
             raise ValueError("run_limit must be > 1")
 
         self.use_ta_time_bound = use_ta_time_bound
+        self.elapsed_time = 0.
 
         # stage variables
-        # the intensification procedure is divided into 3 'stages':
+        # the intensification procedure is divided into 4 'stages':
+        # 0. run 1st configuration (only in the 1st run when incumbent=None)
         # 1. add incumbent run
         # 2. race challenger
         # 3. race against configuration for a new incumbent
-        self.stage = IntensifierStage.RUN_INCUMBENT
-        self.first_run = True
+        self.stage = IntensifierStage.RUN_FIRST_CONFIG
         self.n_iters = 0
 
         # challenger related variables
@@ -172,6 +174,8 @@ class Intensifier(AbstractRacer):
         if time_bound < self._min_time:
             raise ValueError("time_bound must be >= %f" % self._min_time)
 
+        start_time = time.time()
+
         # The "intensification" is designed to be spread across multiple ``eval_challenger()`` runs
         # Lines 1 + 2 happen in the optimizer (SMBO)
 
@@ -182,11 +186,15 @@ class Intensifier(AbstractRacer):
             return incumbent, inc_perf
 
         # if first ever run, then assume current challenger to be the incumbent
-        if self.first_run and not incumbent:
-            self.logger.info("First run, no incumbent provided; challenger is assumed to be the incumbent")
-            incumbent = challenger
-            self.current_challenger = None
-            self.first_run = False
+        if self.stage == IntensifierStage.RUN_FIRST_CONFIG:
+            if incumbent is None:
+                self.logger.info("First run, no incumbent provided; challenger is assumed to be the incumbent")
+                incumbent = challenger
+            else:
+                inc_runs = run_history.get_runs_for_config(incumbent, only_max_observed_budget=True)
+                if len(inc_runs) > 0:
+                    self.logger.debug("Skipping RUN_FIRST_CONFIG stage since incumbent is already run before")
+                    self.stage = IntensifierStage.RUN_INCUMBENT
 
         self.logger.debug("Intensify on %s", challenger)
         if hasattr(challenger, 'origin'):
@@ -195,28 +203,17 @@ class Intensifier(AbstractRacer):
         try:
             # since it runs only 1 "ExecuteRun" per iteration,
             # we run incumbent once and then challenger in the next
-            if self.stage == IntensifierStage.RUN_INCUMBENT:
+            if self.stage == IntensifierStage.RUN_FIRST_CONFIG or \
+                    self.stage == IntensifierStage.RUN_INCUMBENT:
                 # Lines 3-7
-                self._add_inc_run(incumbent=incumbent, run_history=run_history)
-            elif self.stage == IntensifierStage.RUN_CHALLENGER:
+                self._add_inc_run(incumbent=incumbent, run_history=run_history, log_traj=log_traj)
+            elif self.stage == IntensifierStage.RUN_CHALLENGER or \
+                    self.stage == IntensifierStage.RUN_BASIS:
                 # Lines 8-17
                 incumbent = self._race_challenger(challenger=challenger,
                                                   incumbent=incumbent,
                                                   run_history=run_history,
                                                   log_traj=log_traj)
-            else:
-                if self.always_race_against and \
-                        challenger == incumbent and \
-                        self.always_race_against != challenger:
-                    self.logger.debug("Race against constant configuration after incumbent change.")
-                    incumbent = self._race_challenger(challenger=self.always_race_against,
-                                                      incumbent=incumbent,
-                                                      run_history=run_history,
-                                                      log_traj=log_traj)
-                else:
-                    # move on to next iteration
-                    self.stage = IntensifierStage.RUN_INCUMBENT
-                    self.continue_challenger = False
 
         except BudgetExhaustedException:
             # We return incumbent, SMBO stops due to its own budget checks
@@ -224,24 +221,24 @@ class Intensifier(AbstractRacer):
             self.logger.debug("Budget exhausted; Return incumbent")
             return incumbent, inc_perf
 
-        # check if 1 intensification run is complete
-        tm = time.time()
-        if self.stage == IntensifierStage.RUN_INCUMBENT and self._chall_indx >= self.min_chall:
-            if self._num_run > self.run_limit:
-                self.logger.info("Maximum #runs for intensification reached")
-                self._update_trackers()
+        # time used to evaluate challenger
+        self.elapsed_time += time.time() - start_time
 
-            if not self.use_ta_time_bound and tm - self.start_time - time_bound >= 0:
-                self.logger.debug("Wallclock time limit for intensification reached ("
-                                  "used: %f sec, available: %f sec)" %
-                                  (tm - self.start_time, time_bound))
-                self._update_trackers()
+        # check if 1 intensification run is complete - line 18
+        if self.stage == IntensifierStage.RUN_INCUMBENT and self._chall_indx >= self.min_chall:
+            if self.num_run > self.run_limit:
+                self.logger.info("Maximum #runs for intensification reached")
+                self._next_iteration()
+
+            if not self.use_ta_time_bound and self.elapsed_time - time_bound >= 0:
+                self.logger.info("Wallclock time limit for intensification reached (used: %f sec, available: %f sec)",
+                                 self.elapsed_time, time_bound)
+                self._next_iteration()
 
             elif self._ta_time - time_bound >= 0:
-                self.logger.debug("TA time limit for intensification reached ("
-                                  "used: %f sec, available: %f sec)" %
-                                  (self._ta_time, time_bound))
-                self._update_trackers()
+                self.logger.info("TA time limit for intensification reached (used: %f sec, available: %f sec)",
+                                 self._ta_time, time_bound)
+                self._next_iteration()
 
         inc_perf = run_history.get_cost(incumbent)
 
@@ -249,7 +246,8 @@ class Intensifier(AbstractRacer):
 
     def _add_inc_run(self,
                      incumbent: Configuration,
-                     run_history: RunHistory) -> None:
+                     run_history: RunHistory,
+                     log_traj: bool = True) -> None:
         """Add new run for incumbent
 
         *Side effect:* adds runs to <run_history>
@@ -260,6 +258,9 @@ class Intensifier(AbstractRacer):
             best configuration so far
         run_history : smac.runhistory.runhistory.RunHistory
             stores all runs we ran so far
+        log_traj: bool
+            Whether to log changes of incumbents in trajectory
+
         """
         inc_runs = run_history.get_runs_for_config(incumbent, only_max_observed_budget=True)
 
@@ -304,7 +305,7 @@ class Intensifier(AbstractRacer):
                     cutoff=self.cutoff,
                     instance_specific=self.instance_specifics.get(next_instance, "0"))
                 self._ta_time += dur
-                self._num_run += 1
+                self.num_run += 1
             else:
                 self.logger.debug("No further instance-seed pairs for "
                                   "incumbent available.")
@@ -317,14 +318,24 @@ class Intensifier(AbstractRacer):
             self.logger.info("Updated estimated cost of incumbent on %d runs: %.4f"
                              % (len(inc_runs), inc_perf))
 
-            # Termination condition; after each run, this checks
-            # whether further runs are necessary due to minR
-            if len(inc_runs) >= self.minR or len(inc_runs) >= self.maxR:
-                self.stage = IntensifierStage.RUN_CHALLENGER
+            # if running first configuration, go to next stage after 1st run
+            if self.stage == IntensifierStage.RUN_FIRST_CONFIG:
+                self.stage = IntensifierStage.RUN_INCUMBENT
+                self._next_iteration()
+            else:
+                # Termination condition; after each run, this checks
+                # whether further runs are necessary due to minR
+                if len(inc_runs) >= self.minR or len(inc_runs) >= self.maxR:
+                    self.stage = IntensifierStage.RUN_CHALLENGER
         else:
             # maximum runs for incumbent reached, do not run incumbent
             self.stage = IntensifierStage.RUN_CHALLENGER
 
+        self._compare_configs(
+            incumbent=incumbent, challenger=incumbent,
+            run_history=run_history,
+            log_traj=log_traj)
+
     def _race_challenger(self,
                          challenger: Configuration,
                          incumbent: Configuration,
@@ -390,7 +401,7 @@ class Intensifier(AbstractRacer):
                     # Cutoff might be None if self.cutoff is None, but then the first if statement prevents
                     # evaluation of the second if statement
                     capped=(self.cutoff is not None) and (cutoff < self.cutoff))  # type: ignore[operator] # noqa F821
-                self._num_run += 1
+                self.num_run += 1
                 self._ta_time += dur
 
             except CappedRunException:
@@ -402,7 +413,6 @@ class Intensifier(AbstractRacer):
 
         chal_runs = run_history.get_runs_for_config(challenger, only_max_observed_budget=True)
         chal_perf = run_history.get_cost(challenger)
-        self.logger.debug('Estimated cost of challenger on %d runs: %.4f' % (len(chal_runs), chal_perf))
 
         # if all <instance, seed> have been run, compare challenger performance
         if not self.to_run:
@@ -410,20 +420,37 @@ class Intensifier(AbstractRacer):
                 incumbent=incumbent, challenger=challenger,
                 run_history=run_history,
                 log_traj=log_traj)
+
+            # update intensification stage
             if new_incumbent == incumbent:
                 # move on to the next iteration
                 self.stage = IntensifierStage.RUN_INCUMBENT
                 self.continue_challenger = False
+                self.logger.debug('Estimated cost of challenger on %d runs: %.4f, but worse than incumbent',
+                                  len(chal_runs), chal_perf)
 
             elif new_incumbent == challenger:
+                # New incumbent found
                 incumbent = challenger
-                # New incumbent found. Go to the next stage
-                self.stage = IntensifierStage.RUN_DEFAULT
+                self.continue_challenger = False
+                # compare against basis configuration if provided, else go to next iteration
+                if self.always_race_against and \
+                        self.always_race_against != challenger:
+                    self.stage = IntensifierStage.RUN_BASIS
+                else:
+                    self.stage = IntensifierStage.RUN_INCUMBENT
+                self.logger.debug('Estimated cost of challenger on %d runs: %.4f, becomes new incumbent',
+                                  len(chal_runs), chal_perf)
 
             else:  # Line 17
                 # challenger is not worse, continue
                 self.N = 2 * self.N
                 self.continue_challenger = True
+                self.logger.debug('Estimated cost of challenger on %d runs: %.4f, adding %d runs to the queue',
+                                  len(chal_runs), chal_perf, self.N / 2)
+        else:
+            self.logger.debug('Estimated cost of challenger on %d runs: %.4f, still %d runs to go (continue racing)',
+                              len(chal_runs), chal_perf, len(self.to_run))
 
         return incumbent
 
@@ -459,7 +486,7 @@ class Intensifier(AbstractRacer):
         inc_inst_seeds = set(run_history.get_runs_for_config(incumbent, only_max_observed_budget=True))
         chall_inst_seeds = set(run_history.get_runs_for_config(challenger, only_max_observed_budget=True))
         # Line 10
-        missing_runs = list(inc_inst_seeds - chall_inst_seeds)
+        missing_runs = sorted(inc_inst_seeds - chall_inst_seeds)
 
         # Line 11
         self.rs.shuffle(missing_runs)
@@ -512,6 +539,11 @@ class Intensifier(AbstractRacer):
         # sampling from next challenger marks the beginning of a new iteration
         self.iteration_done = False
 
+        # if in RUN_BASIS stage, return the basis configuration (i.e., `always_race_against`)
+        if self.stage == IntensifierStage.RUN_BASIS:
+            self.logger.debug("Race against basis configuration after incumbent change.")
+            return self.always_race_against, False
+
         # if the current challenger could not be rejected, it is run again on more instances
         if self.current_challenger and self.continue_challenger:
             return self.current_challenger, False
@@ -532,7 +564,7 @@ class Intensifier(AbstractRacer):
                 challenger = next(self.configs_to_run)
             except StopIteration:
                 # out of challengers for the current iteration, start next incumbent iteration
-                self._update_trackers()
+                self._next_iteration()
                 return None, False
 
             if challenger:
@@ -542,10 +574,6 @@ class Intensifier(AbstractRacer):
                 self.N = max(1, self.minR)
                 self.to_run = []
 
-                # reset time bound related params since this is a new configuration
-                self.start_time = time.time()
-                self._ta_time = 0.0
-
             return challenger, True
 
         # return currently running challenger
@@ -585,7 +613,7 @@ class Intensifier(AbstractRacer):
 
         return chall_gen
 
-    def _update_trackers(self) -> None:
+    def _next_iteration(self) -> None:
         """
         Updates tracking variables at the end of an intensification run
         """
@@ -596,8 +624,10 @@ class Intensifier(AbstractRacer):
         self.update_configs_to_run = True
 
         # reset for a new iteration
-        self._num_run = 0
+        self.num_run = 0
         self._chall_indx = 0
+        self.elapsed_time = 0
+        self._ta_time = 0.0
 
         self.stats.update_average_configs_per_intensify(
             n_configs=self._chall_indx)
