@@ -29,6 +29,11 @@ class StatusType(Enum):
     ABORT = 4
     MEMOUT = 5
     CAPPED = 6
+    # Only relevant for SH/HB. Run might have a results, but should not be considered further.
+    # By default, these runs will always be considered for building the model. Potential use cases:
+    # 1) The run has converged and does not benefit from a higher budget
+    # 2) The run has exhausted given resources and will not benefit from higher budgets
+    DONOTADVANCE = 7
 
     @staticmethod
     def enum_hook(obj: typing.Dict) -> typing.Any:
@@ -193,7 +198,9 @@ class ExecuteTARun(object):
                                                           seed=seed,
                                                           budget=budget,
                                                           instance_specific=instance_specific)
-
+        if budget == 0 and status == StatusType.DONOTADVANCE:
+            raise ValueError("Cannot handle DONOTADVANCE state when using intensify or SH/HB on "
+                             "instances.")
         # update SMAC stats
         self.stats.ta_runs += 1
         self.stats.ta_time_used += float(runtime)
