@@ -15,7 +15,7 @@ __license__ = "3-clause BSD"
 
 TrajEntry = collections.namedtuple(
     'TrajEntry', ['train_perf', 'incumbent_id', 'incumbent',
-                  'ta_runs', 'ta_time_used', 'wallclock_time'])
+                  'ta_runs', 'ta_time_used', 'wallclock_time', 'budget'])
 
 
 class TrajLogger(object):
@@ -74,8 +74,10 @@ class TrajLogger(object):
 
         self.trajectory = []  # type: typing.List[TrajEntry]
 
-    def add_entry(self, train_perf: float, incumbent_id: int,
-                  incumbent: Configuration) -> None:
+    def add_entry(self, train_perf: float,
+                  incumbent_id: int,
+                  incumbent: Configuration,
+                  budget: float = 0) -> None:
         """Adds entries to trajectory files (several formats) with using the
         same timestamps for each entry
 
@@ -87,22 +89,25 @@ class TrajLogger(object):
             id of incumbent
         incumbent: Configuration()
             current incumbent configuration
+        budget: float
+            budget used in intensifier to limit TA (default: 0)
         """
         ta_runs = self.stats.ta_runs
         ta_time_used = self.stats.ta_time_used
         wallclock_time = self.stats.get_used_wallclock_time()
         self.trajectory.append(TrajEntry(train_perf, incumbent_id, incumbent,
-                               ta_runs, ta_time_used, wallclock_time))
+                               ta_runs, ta_time_used, wallclock_time, budget))
         if self.output_dir is not None:
             self._add_in_old_format(train_perf, incumbent_id, incumbent,
                                     ta_time_used, wallclock_time)
             self._add_in_aclib_format(train_perf, incumbent_id, incumbent,
                                       ta_time_used, wallclock_time)
-            self._add_in_alljson_format(train_perf, incumbent_id, incumbent,
+            self._add_in_alljson_format(train_perf, incumbent_id, incumbent, budget,
                                         ta_time_used, wallclock_time)
 
     def _add_in_old_format(self, train_perf: float, incumbent_id: int,
-                           incumbent: Configuration, ta_time_used: float,
+                           incumbent: Configuration,
+                           ta_time_used: float,
                            wallclock_time: float) -> None:
         """Adds entries to old SMAC2-like trajectory file
 
@@ -136,7 +141,8 @@ class TrajLogger(object):
             ))
 
     def _add_in_aclib_format(self, train_perf: float, incumbent_id: int,
-                             incumbent: Configuration, ta_time_used: float,
+                             incumbent: Configuration,
+                             ta_time_used: float,
                              wallclock_time: float) -> None:
         """Adds entries to AClib2-like trajectory file
 
@@ -172,7 +178,8 @@ class TrajLogger(object):
             fp.write("\n")
 
     def _add_in_alljson_format(self, train_perf: float, incumbent_id: int,
-                               incumbent: Configuration, ta_time_used: float,
+                               incumbent: Configuration, budget: float,
+                               ta_time_used: float,
                                wallclock_time: float) -> None:
         """Adds entries to AClib2-like (but with configs as json) trajectory file
 
@@ -184,6 +191,8 @@ class TrajLogger(object):
             Id of incumbent
         incumbent: Configuration()
             Current incumbent configuration
+        budget: float
+            budget (cutoff) used in intensifier to limit TA (default: 0)
         ta_time_used: float
             CPU time used by the target algorithm
         wallclock_time: float
@@ -194,6 +203,7 @@ class TrajLogger(object):
                       "evaluations": self.stats.ta_runs,
                       "cost": train_perf,
                       "incumbent": incumbent.get_dictionary(),
+                      "budget": budget,
                       "origin": incumbent.origin,
                       }
 
@@ -224,6 +234,7 @@ class TrajLogger(object):
             "wallclock_time": float,
             "evaluations": int
             "cost": float,
+            "budget": budget,
             "incumbent": Configuration
             }
         """
