@@ -2,9 +2,9 @@ from ConfigSpace.configuration_space import ConfigurationSpace
 from ConfigSpace.hyperparameters import UniformFloatHyperparameter
 
 
-def get_branin_config_space():
+def get_branin_config_space() -> ConfigurationSpace:
     """TODO"""
     cs = ConfigurationSpace()
     cs.add_hyperparameter(UniformFloatHyperparameter('x', -5, 10))
     cs.add_hyperparameter(UniformFloatHyperparameter('y', 0, 15))
-    return cs
\ No newline at end of file
+    return cs
